#include <X11/Xlib.h>
#include <X11/Xmu/WinUtil.h>
#include <X11/Xatom.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static Atom _MOZILLA_LOCK, _MOZILLA_COMMAND, _MOZILLA_VERSION, _MOZILLA_RESPONSE;

static void   create_atoms(Display *);
static Window find_netscape(Display *, int, int);
static void   lock_netscape(Display *, Window);
static void   netscape_command(Display *, Window, char *);
static void   unlock_netscape(Display *, Window);

void main() {
  Display *dpy;
  Window  root, netsc;

  dpy = XOpenDisplay(NULL);
  create_atoms(dpy);
  netsc = find_netscape(dpy, 1, 12);
  if (netsc == (Window)0) {
    if (system("netscape & ") == 0) {
      sleep(20);
      netsc = find_netscape(dpy, 1, 12);
    }
    if (netsc == (Window)0) {
      fprintf(stderr,
	      "Couldn't find Netscape. Check your PATH variable.\n");
      exit(1);
    }
  }
  netscape_command(dpy, netsc, 
		   "openURL(file:/usr/doc/calderadoc-0.80-1/Caldera_Info)");
}

static void create_atoms (Display *dpy)
{
  _MOZILLA_VERSION = XInternAtom(dpy,"_MOZILLA_VERSION",  False);
  _MOZILLA_LOCK    = XInternAtom(dpy,"_MOZILLA_LOCK",     False);
  _MOZILLA_COMMAND = XInternAtom(dpy,"_MOZILLA_COMMAND",  False);
  _MOZILLA_RESPONSE= XInternAtom(dpy,"_MOZILLA_RESPONSE", False);
}


static Window find_netscape(Display *dpy, int major, int minor) {

  register int  i;

  Window        ignore, *children, cw, w;
  unsigned int  childCount;
  int           actualFormat;
  int           majorFound, minorFound;
  unsigned long itemCount, trailingBytes;
  unsigned char *property;    
  Atom          actualType;


  XQueryTree(dpy, DefaultRootWindow(dpy), &ignore, &ignore, 
	     &children, &childCount);
  i = 0;
  w = (Window)0;
  while(i < childCount && w == (Window)0) {
    /*
     * XmuClientWindow() besorgt das "echte" Fenster der Applikation,
     * whrend XQueryTree nur die vom Window-Manager angelegten 
     * Umrahmungen findet. Die gesuchte Property existiert auf dem
     * "echten" Netscape-Fenster
     */
    cw = XmuClientWindow (dpy, children[i]);

    if (XGetWindowProperty(dpy, cw, _MOZILLA_VERSION, 0, 50, 
			   False, XA_STRING,
			   &actualType, &actualFormat, 
			   &itemCount, &trailingBytes,
			   &property) == Success && 
	itemCount > 0 && property != NULL) {
      majorFound = atoi(property);
      minorFound = atoi(strchr(property,'.')+1);
      XFree(property);
      if (majorFound > major || 
	  (majorFound == major && minorFound >= minor))
	w = cw;
      else {
	fprintf(stderr,
		"Netscape Version %s doesn't match required %1d.%1d\n",
		property, major, minor);
	exit(1);
      }
    } 
    i++;
  } /* while */
  XFree(children);
  return w;
}

static void netscape_command(Display *dpy, Window w, char *command)
{
  int state = 0;
  XEvent event;

  XSelectInput(dpy, w, PropertyChangeMask);
  /*
   * Netscape reservieren
   */
  lock_netscape(dpy, w);

  /*
   * Kommando schicken
   */
  XChangeProperty (dpy, w, _MOZILLA_COMMAND, XA_STRING, 8,
		   PropModeReplace, 
		   (unsigned char *)command, strlen (command));
  
  while (1) {
    XNextEvent(dpy, &event) ;
      
    /*
     * Warten, bis eine Antwort da ist
     */
    if(event.type == PropertyNotify &&
       event.xproperty.state == PropertyNewValue &&
       event.xproperty.window == w &&
       event.xproperty.atom == _MOZILLA_RESPONSE) {
      Atom actual_type;
      int actual_format, result;
      unsigned long nitems, bytes_after;
      unsigned char *data = 0;
      
      result = XGetWindowProperty (dpy, w, _MOZILLA_RESPONSE,
				   0, (65536 / sizeof (long)),
				   True, /* atomic delete after */
				   XA_STRING,
				   &actual_type, &actual_format,
				   &nitems, &bytes_after,
				   &data);
      if (result == Success) {
	if (strncmp(data,"500",3) == 0 || strncmp(data,"501",3) == 0)
	  fprintf(stderr,"Kommando nicht verstanden: %s\n", data+4);
      }  
      break;
    }
   }
   unlock_netscape(dpy, w);
}


/* -----------------------------------------------------------
   Stark vereinfachte Funktion zum Locken des Viewers. Scheitert,
   wenn ein anderes Programm bereits einen Lock hat.
   ----------------------------------------------------------- */

static void lock_netscape(Display *dpy, Window w)
{
  static char *lock_data;
  int result;
  Atom actual_type;
  int actual_format;
  unsigned long nitems, bytes_after;
  unsigned char *data = 0;

  if (! lock_data)
    {
      lock_data = (char *) malloc (255);
      sprintf (lock_data, "pid%d@", getpid ());
      gethostname (lock_data + strlen (lock_data), 100);
    }

  XGrabServer (dpy);   
  result = XGetWindowProperty (dpy, w, _MOZILLA_LOCK,
			       0, (65536 / sizeof (long)),
			       False, /* don't delete */
			       XA_STRING,
			       &actual_type, &actual_format,
			       &nitems, &bytes_after,
			       &data);
  if (result != Success || actual_type == None) {
    /*
     * Kein Lock gefunden -- selber locken
     */
      XChangeProperty (dpy, w, _MOZILLA_LOCK, XA_STRING, 8,
		       PropModeReplace, 
		       (unsigned char *)lock_data, strlen (lock_data));
    }
  XUngrabServer (dpy); 
  XSync (dpy, False);
}

static void unlock_netscape(Display *dpy, Window w)
{
  int result;
  Atom actual_type;
  int actual_format;
  unsigned long nitems, bytes_after;
  unsigned char *data = 0;

  result = XGetWindowProperty (dpy, w, _MOZILLA_LOCK,
			       0, (65536 / sizeof (long)),
			       True, /* atomic delete after */
			       XA_STRING,
			       &actual_type, &actual_format,
			       &nitems, &bytes_after,
			       &data);
} 

